package jp.co.ois.sample.rabbitmqconsumer;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.boot.autoconfigure.amqp.SimpleRabbitListenerContainerFactoryConfigurer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Configuration
public class HogeConfig {

    @Bean
    public SimpleRabbitListenerContainerFactory myFactory(
            ConnectionFactory connectionFactory,
            SimpleRabbitListenerContainerFactoryConfigurer configurer) {
        SimpleRabbitListenerContainerFactory factory = new SimpleRabbitListenerContainerFactory();
        configurer.configure(factory, connectionFactory);
        ExecutorService executor = Executors.newCachedThreadPool(new HogeThreadFactory("amqp-%d"));
        factory.setTaskExecutor(executor);
        return factory;
    }

    public static class HogeThreadFactory implements ThreadFactory {
        private final AtomicInteger counter = new AtomicInteger(0);
        private final String format;

        private HogeThreadFactory(String format) {
            this.format = format;
        }

        @Override
        public Thread newThread(Runnable r) {
            String name = String.format(format, counter.incrementAndGet());
            return new Thread(null, r, name);
        }
    }
}
